/*
 *  This file is part of the KDE project
 *  Copyright (C) 2002 Ian Reinhart Geiser <geiseri@kde.org>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License version 2.0 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 *
 *  $Id$
 */

#ifndef __SIDEBARWIDGET_H__
#define __SIDEBARWIDGET_H__

#include <tqwidget.h>

class TQVBox;
class TQVBoxLayout;
class TQScrollView;
class KCMultiDialog;
class DCOPRef;

class sidebarwidget : public TQWidget
{
TQ_OBJECT

  public:
    sidebarwidget(TQWidget* parent, const char* name = 0);
    virtual ~sidebarwidget();
    TQWidget *viewport();
    TQScrollView *reportGrid;
    TQVBoxLayout *mainLayout;
    TQVBoxLayout *layout;
    TQVBox *view;


  private slots:
    void slotRefresh();
    void slotConfigure();

  private:
    DCOPRef *dcopCall;
    KCMultiDialog *settingsDialog;

};

#endif


